@@ -246,7 +246,7 @@ size_t basic::nops() const
 }
 
 /** Return operand/member at position i. */
-ex basic::op(size_t /*unused*/) const
+const ex basic::op(size_t /*unused*/) const
 {
 	throw(std::range_error(std::string("basic::op(): ") + class_name() + std::string(" has no operands")));
 }
@@ -591,9 +591,14 @@ bool basic::match(const ex & pattern, lst & repl_lst) const
 /** Helper function for subs(). Does not recurse into subexpressions. */
 ex basic::subs_one_level(const exmap & m, unsigned options) const
 {
-	if ((options & subs_options::no_pattern) != 0u) {
-		const auto& it = m.find(*this);
-		return (it != m.end())? it->second : *this;
+	exmap::const_iterator it;
+
+        if (options & subs_options::no_pattern) {
+		ex thisex = *this;
+		it = m.find(thisex);
+		if (it != m.end())
+			return it->second;
+		return thisex;
 	} else {
                 for (const auto & elem : m) {
 			lst repl_lst;
@@ -770,7 +775,7 @@ tinfo_t basic::return_type_tinfo() const
  *  would all end up with the same hashvalue. */
 long basic::calchash() const
 {
-	long v = golden_ratio_hash((p_int)tinfo());
+	long v = golden_ratio_hash((intptr_t)tinfo());
 	for (size_t i=0; i<nops(); i++) {
 		v = rotate_left(v);
 		v ^= this->op(i).gethash();
