@@ -96,6 +96,8 @@ ex constant::unarchive(const archive_node &n, lst &sym_lst)
 			return Catalan;
 		else if (s == Euler.name)
 			return Euler;
+		else if (s == NaN.name)
+			return NaN;
 		else {
 			ans = py_funcs.py_get_constant(s.c_str());
 			if (PyErr_Occurred() != nullptr) {
@@ -233,7 +235,7 @@ bool constant::is_equal_same_type(const basic & other) const
 
 long constant::calchash() const
 {
-	hashvalue = golden_ratio_hash((p_int)tinfo() ^ serial);
+	hashvalue = golden_ratio_hash((intptr_t)tinfo() ^ serial);
 
 	setflag(status_flags::hash_calculated);
 
@@ -273,4 +275,8 @@ const constant Euler("euler_gamma", ConstantEvalf, "\\gamma_E", domain::positive
  *  Calls python function py_eval_constant for evalf(). */
 const constant Catalan("catalan", ConstantEvalf, "G", domain::positive);
 
+/** Not a number
+ *  Calls python function py_eval_constant for evalf(). */
+const constant NaN("NaN", ConstantEvalf, "NaN", domain::complex);
+
 } // namespace GiNaC
