@@ -141,6 +141,12 @@ function_options & function_options::eval_func(eval_funcp_3 e)
 	eval_f = eval_funcp(e);
 	return *this;
 }
+function_options & function_options::eval_func(eval_funcp_6 e)
+{
+	test_and_set_nparams(6);
+	eval_f = eval_funcp(e);
+	return *this;
+}
 
 function_options & function_options::evalf_func(evalf_funcp_1 ef)
 {
@@ -160,6 +166,12 @@ function_options & function_options::evalf_func(evalf_funcp_3 ef)
 	evalf_f = evalf_funcp(ef);
 	return *this;
 }
+function_options & function_options::evalf_func(evalf_funcp_6 ef)
+{
+	test_and_set_nparams(6);
+	evalf_f = evalf_funcp(ef);
+	return *this;
+}
 
 function_options & function_options::conjugate_func(conjugate_funcp_1 c)
 {
@@ -236,6 +248,12 @@ function_options & function_options::derivative_func(derivative_funcp_3 d)
 	derivative_f = derivative_funcp(d);
 	return *this;
 }
+function_options & function_options::derivative_func(derivative_funcp_6 d)
+{
+	test_and_set_nparams(6);
+	derivative_f = derivative_funcp(d);
+	return *this;
+}
 
 function_options & function_options::power_func(power_funcp_1 d)
 {
@@ -515,6 +533,11 @@ function::function(unsigned ser, const ex & param1, const ex & param2, const ex
 {
 	tinfo_key = &function::tinfo_static;
 }
+function::function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5, const ex & param6)
+	: exprseq(param1, param2, param3, param4, param5, param6), serial(ser)
+{
+	tinfo_key = &function::tinfo_static;
+}
 
 // end of generated lines
 
@@ -928,7 +951,7 @@ ex function::evalf(int level, PyObject* kwds) const
 
 long function::calchash() const
 {
-	long v = golden_ratio_hash(golden_ratio_hash((p_int)tinfo()) ^ serial);
+	long v = golden_ratio_hash(golden_ratio_hash((intptr_t)tinfo()) ^ serial);
 	for (size_t i=0; i<nops(); i++) {
 		v = rotate_left(v);
 		v ^= this->op(i).gethash();
