@@ -174,8 +174,8 @@ static ex zeta1_series(const ex& m, const relational& rel, int order, unsigned o
 	ex ser = 1/(m-1);
 	numeric fac = 1;
 	for (int n = 0; n <= order; ++n) {
-	  fac = fac.mul(n+1);
 	  ser += pow(-1, n) * stieltjes(n) * pow(m-1, n) * fac.inverse();
+	  fac = fac.mul(n+1);
 	}
 	return ser.series(rel, order, options);
 }
@@ -356,4 +356,35 @@ unsigned zeta2_SERIAL::serial = function::register_new(function_options("zeta",
                                 print_func<print_latex>(zeta2_print_latex).
                                 overloaded(2));
 
+//////////
+// Derivatives of Riemann's Zeta-function  zetaderiv(0,x)==zeta(x)
+//////////
+
+static ex zetaderiv_eval(const ex & n, const ex & x)
+{
+	if (n.info(info_flags::numeric)) {
+		// zetaderiv(0,x) -> zeta(x)
+		if (n.is_zero())
+			return zeta(x).hold();
+	}
+	
+	return zetaderiv(n, x).hold();
+}
+
+static ex zetaderiv_deriv(const ex & n, const ex & x, unsigned deriv_param)
+{
+	GINAC_ASSERT(deriv_param<2);
+	
+	if (deriv_param==0) {
+		// d/dn zeta(n,x)
+		throw(std::logic_error("cannot diff zetaderiv(n,x) with respect to n"));
+	}
+	// d/dx psi(n,x)
+	return zetaderiv(n+1,x);
+}
+
+REGISTER_FUNCTION(zetaderiv, eval_func(zetaderiv_eval).
+	                       	 derivative_func(zetaderiv_deriv).
+  	                         latex_name("\\zeta^\\prime"));
+
 } // namespace GiNaC
