@@ -205,7 +205,14 @@ size_t relational::nops() const
 	return 2;
 }
 
-ex relational::op(size_t i) const
+const ex relational::op(size_t i) const
+{
+	GINAC_ASSERT(i<2);
+
+	return i==0 ? lh : rh;
+}
+
+ex & relational::let_op(size_t i)
 {
 	GINAC_ASSERT(i<2);
 
@@ -308,7 +315,7 @@ tinfo_t relational::return_type_tinfo() const
 
 long relational::calchash() const
 {
-	long v = golden_ratio_hash((p_int)tinfo());
+	long v = golden_ratio_hash((intptr_t)tinfo());
 	long lhash = lh.gethash();
 	long rhash = rh.gethash();
 
